@@ -77,13 +77,18 @@ double convert_to_double(long long a) {
 }
 
 template<typename Integer>
-void bottom_points(list< vector<Integer> >& new_points, const Matrix<Integer>& gens,Integer VolumeBound) {
+void bottom_points(list< vector<Integer> >& new_points, const Matrix<Integer>& given_gens,Integer VolumeBound) {
     
     /* gens.pretty_print(cout);
     cout << "=======================" << endl;
     
     gens.transpose().pretty_print(cout);
     cout << "=======================" << endl;*/
+    
+    Matrix<Integer> gens, Trans, Trans_inv;
+    // given_gens.LLL_transform_transpose(gens,Trans,Trans_inv);  // now in optimal_subdivision_point()
+    gens=given_gens;
+    
     Integer volume;
     // int dim = gens[0].size();
     Matrix<Integer> Support_Hyperplanes = gens.invert(volume);
@@ -205,6 +210,7 @@ void bottom_points(list< vector<Integer> >& new_points, const Matrix<Integer>& g
         #pragma omp flush(skip_remaining)
     }
 #endif
+
     } // end parallel
     
 //---------------------------- end stellar subdivision -----------------------
@@ -221,8 +227,13 @@ void bottom_points(list< vector<Integer> >& new_points, const Matrix<Integer>& g
         verboseOutput() << new_points.size() << " bottom points accumulated in total." << endl;
         verboseOutput() << "The sum of determinants of the stellar subdivision is " << stellar_det_sum << endl;
     }
+    
+    /* for(auto it=new_points.begin();it!=new_points.end();++it)
+        *it=Trans_inv.VxM(*it); */
 }
 
+//-----------------------------------------------------------------------------------------
+
 template<typename Integer>
 bool bottom_points_inner(SCIP* scip, Matrix<Integer>& gens, list< vector<Integer> >& local_new_points,
                  vector< Matrix<Integer> >& local_q_gens, size_t& stellar_det_sum) {
