@@ -113,6 +113,10 @@ ConeProperties& ConeProperties::reset_compute_options() {
     CPs.set(ConeProperty::NoNestedTri, false);
     CPs.set(ConeProperty::NoPeriodBound, false);
     CPs.set(ConeProperty::SCIP, false);
+    CPs.set(ConeProperty::NoLLL, false);
+    CPs.set(ConeProperty::NoRelax, false);
+    CPs.set(ConeProperty::ExplicitHilbertSeries, false);
+    CPs.set(ConeProperty::NakedDual, false);
     return *this;
 }
 
@@ -144,6 +148,10 @@ ConeProperties ConeProperties::options() {
     ret.set(ConeProperty::BigInt, CPs.test(ConeProperty::BigInt));
     ret.set(ConeProperty::NoPeriodBound, CPs.test(ConeProperty::NoPeriodBound));
     ret.set(ConeProperty::SCIP, CPs.test(ConeProperty::SCIP));
+    ret.set(ConeProperty::NoLLL, CPs.test(ConeProperty::NoLLL));
+    ret.set(ConeProperty::NoRelax, CPs.test(ConeProperty::NoRelax));
+    ret.set(ConeProperty::ExplicitHilbertSeries, CPs.test(ConeProperty::ExplicitHilbertSeries));
+    ret.set(ConeProperty::NakedDual, CPs.test(ConeProperty::NakedDual));
     return ret;
 }
 
@@ -165,6 +173,19 @@ size_t ConeProperties::count() const {
 /* add preconditions */
 void ConeProperties::set_preconditions(bool inhomogeneous) {
     
+    if(CPs.test(ConeProperty::EuclideanVolume))
+        CPs.set(ConeProperty::Volume);
+    
+    if(inhomogeneous && CPs.test(ConeProperty::Deg1Elements)){
+        CPs.set(ConeProperty::ModuleGenerators);
+        CPs.reset(ConeProperty::Deg1Elements);
+    }
+    
+    if(!inhomogeneous && CPs.test(ConeProperty::Volume)){
+        CPs.set(ConeProperty::Multiplicity);
+    }
+        
+    
     if(CPs.test(ConeProperty::VerticesFloat)){
         CPs.set(ConeProperty::SupportHyperplanes);
         if(!inhomogeneous)
@@ -332,10 +353,12 @@ void ConeProperties::check_conflicting_variants() {
         nr_var++;
     if(CPs.test(ConeProperty::Projection))
         nr_var++;
+    if(CPs.test(ConeProperty::ProjectionFloat))
+        nr_var++;
     if(CPs.test(ConeProperty::Approximate))
         nr_var++;
     if(nr_var>1)
-        throw BadInputException("Only one of DualMode, PrimalMode, Approximate, Projection allowed.");
+        throw BadInputException("Only one of DualMode, PrimalMode, Approximate, Projection, ProjectionFloat allowed.");
 }
 
 void ConeProperties::check_sanity(bool inhomogeneous) {    
@@ -398,6 +421,8 @@ namespace {
         CPN.at(ConeProperty::TriangulationDetSum) = "TriangulationDetSum";
         CPN.at(ConeProperty::Triangulation) = "Triangulation";
         CPN.at(ConeProperty::Multiplicity) = "Multiplicity";
+        CPN.at(ConeProperty::Volume) = "Volume";
+        CPN.at(ConeProperty::EuclideanVolume) = "EuclideanVolume";
         CPN.at(ConeProperty::RecessionRank) = "RecessionRank";
         CPN.at(ConeProperty::AffineDim) = "AffineDim";
         CPN.at(ConeProperty::ModuleRank) = "ModuleRank";
@@ -427,6 +452,7 @@ namespace {
         CPN.at(ConeProperty::DualMode) = "DualMode";
         CPN.at(ConeProperty::KeepOrder) = "KeepOrder";
         CPN.at(ConeProperty::IntegerHull) = "IntegerHull";
+        CPN.at(ConeProperty::ProjectCone) = "ProjectCone";
         CPN.at(ConeProperty::MaximalSubspace) = "MaximalSubspace";
         CPN.at(ConeProperty::ConeDecomposition) = "ConeDecomposition";
         CPN.at(ConeProperty::HSOP) = "HSOP";
@@ -459,10 +485,14 @@ namespace {
         CPN.at(ConeProperty::IsGorenstein) = "IsGorenstein";
         CPN.at(ConeProperty::NoPeriodBound) = "NoPeriodBound";
         CPN.at(ConeProperty::SCIP) = "SCIP";
+        CPN.at(ConeProperty::NoLLL) = "NoLLL";
+        CPN.at(ConeProperty::NoRelax) = "NoRelax";
         CPN.at(ConeProperty::GeneratorOfInterior) = "GeneratorOfInterior";
+        CPN.at(ConeProperty::ExplicitHilbertSeries) = "ExplicitHilbertSeries";
+        CPN.at(ConeProperty::NakedDual) = "NakedDual";
         
         // detect changes in size of Enum, to remember to update CPN!
-        static_assert (ConeProperty::EnumSize == 71,
+        static_assert (ConeProperty::EnumSize == 78,
             "ConeProperties Enum size does not fit! Update cone_property.cpp!");
         // assert all fields contain an non-empty string
         for (size_t i=0;  i<ConeProperty::EnumSize; i++) {
