@@ -520,6 +520,11 @@ void Output<Integer>::write_inv_file() const{
                 inv <<"integer multiplicity = "      << mult.get_num() << endl;
                 inv <<"integer multiplicity_denom = "<< mult.get_den() << endl;
             }
+            if (Result->isComputed(ConeProperty::Volume)){
+                mpq_class vol = Result->getVolume();
+                inv <<"integer volume = "      << vol.get_num() << endl;
+                inv <<"integer volume_denom = "<< vol.get_den() << endl;
+            }
             if (Result->isComputed(ConeProperty::WeightedEhrhartSeries)){
                 const HilbertSeries& HS=Result->getIntData().getWeightedEhrhartSeries().first;
                 
@@ -539,6 +544,12 @@ void Output<Integer>::write_inv_file() const{
                     inv << "integer weighted_ehrhart_quasipolynomial_denom = " 
                         << Result->getIntData().getWeightedEhrhartQuasiPolynomialDenom() << endl;
                 }
+                if(HS.get_expansion_degree()>-1){
+                    vector<mpz_class> expansion=HS.getExpansion();
+                    inv << "vector weighted_ehrhart_series_expansion " << expansion.size() << " = " << expansion;
+                    inv <<"integer expansion_coeff_common_denom = " << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+                }
+                
             }
             
             if (Result->isComputed(ConeProperty::VirtualMultiplicity)){
@@ -579,6 +590,10 @@ void Output<Integer>::write_inv_file() const{
                     inv << "integer hilbert_quasipolynomial_denom = " 
                         << HS.getHilbertQuasiPolynomialDenom() << endl;
                 }
+                if(HS.get_expansion_degree()>-1){
+                    vector<mpz_class> expansion=HS.getExpansion();
+                    inv << "vector Hilbert_series_expansion " << expansion.size() << " = " << expansion;
+                }
             }
         }
         if (Result->isComputed(ConeProperty::IsReesPrimary)) {
@@ -631,9 +646,19 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const{
     if (HS.getShift() != 0) {
         out << "shift = " << HS.getShift() << endl << endl;
     }
-    out << endl;
     out << "degree of weighted Ehrhart series as rational function = "
             << HS.getDegreeAsRationalFunction() << endl << endl;
+            
+    if(HS.get_expansion_degree()>-1){
+        vector<mpz_class> expansion=HS.getExpansion();
+        out << "Expansion of weighted Ehrhart series" << endl;
+        for(size_t i=0;i<expansion.size();++i)
+            out << i+HS.getShift()  << ": " << expansion[i] << endl;
+        out << "Common denominator of coefficients: ";
+        out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+        out << endl;
+    }
+    
     long period = HS.getPeriod();
     if (period == 1) {
         out << "Weighted Ehrhart polynomial:" << endl;
@@ -688,18 +713,6 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const{
     }
 }
 
-//---------------------------------------------------------------------------
-template<typename Integer>
-void Output<Integer>::write_float(ofstream& out, const Matrix<nmz_float>& mat, size_t nr, size_t nc) const{
-
-    for(size_t i=0;i<nr;++i){
-        for(size_t j=0; j< nc;++j){
-            out << std::setw(10) << mat[i][j];
-        }
-        out << endl;
-    }
-}
-
 //---------------------------------------------------------------------------
 
 template<typename Integer>
@@ -886,7 +899,12 @@ void Output<Integer>::write_files() const {
         if ( Result->isComputed(ConeProperty::Multiplicity) ) {
             out << "multiplicity = "<< Result->getMultiplicity() << endl;
         }
-        if ( Result->isComputed(ConeProperty::ModuleRank) || Result->isComputed(ConeProperty::Multiplicity)) {
+        if ( Result->isComputed(ConeProperty::Volume) && Result->isComputed(ConeProperty::Sublattice)) {
+            out << "volume (normalized) = "<< Result->getVolume() << endl;            
+            out << "volume (Euclidean) = "<< Result->getEuclideanVolume() << endl;
+        }
+        if ( Result->isComputed(ConeProperty::ModuleRank) || Result->isComputed(ConeProperty::Multiplicity) 
+                    || Result->isComputed(ConeProperty::Volume)) {
             out << endl;
         }
         
@@ -919,6 +937,13 @@ void Output<Integer>::write_files() const {
             if(v_is_symmetric(HS_Num)){
                 out << "The numerator of the Hilbert Series is symmetric." << endl << endl;
             }
+            if(HS.get_expansion_degree()>-1){
+                vector<mpz_class> expansion=HS.getExpansion();
+                out << "Expansion of Hilbert series" << endl;
+                for(size_t i=0;i<expansion.size();++i)
+                    out << i+HS.getShift()  << ": " << expansion[i] << endl;
+                out << endl;
+            }
             long period = HS.getPeriod();
             if (period == 1 && (HS_Denom.size() == 0
                                 || HS_Denom.begin()->first== (long) HS_Denom.size())) {
@@ -1089,7 +1114,7 @@ void Output<Integer>::write_files() const {
         if (Result->isComputed(ConeProperty::VerticesOfPolyhedron)  && !no_ext_rays_output) {
             out << Result->getNrVerticesOfPolyhedron() <<" vertices of polyhedron:" << endl;
             if(Result->isComputed(ConeProperty::VerticesFloat))
-                write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim);
+                Result->getVerticesFloatMatrix().pretty_print(out); // write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim);
             else
                 Result->getVerticesOfPolyhedronMatrix().pretty_print(out);
             out << endl;
@@ -1097,7 +1122,7 @@ void Output<Integer>::write_files() const {
         if (Result->isComputed(ConeProperty::ExtremeRays) && !no_ext_rays_output) {
             out << Result->getNrExtremeRays() << " extreme rays" << of_cone << ":" << endl;
             if(homogeneous && Result->isComputed(ConeProperty::VerticesFloat))
-                write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim); 
+                Result->getVerticesFloatMatrix().pretty_print(out); // write_float(out,Result->getVerticesFloatMatrix(),Result->getNrVerticesFloat(),dim); 
             else
                 Result->getExtremeRaysMatrix().pretty_print(out);
             out << endl;
@@ -1142,7 +1167,9 @@ void Output<Integer>::write_files() const {
         if (Result->isComputed(ConeProperty::Sublattice)) {
             const Sublattice_Representation<Integer>& BasisChange = Result->getSublattice();
             //equations
-            const Matrix<Integer>& Equations = BasisChange.getEquationsMatrix();
+            const Matrix<Integer>& EQ = BasisChange.getEquationsMatrix();
+            Matrix<Integer> Equations=EQ;
+            Equations.row_echelon_reduce();
             size_t nr_of_equ = Equations.nr_of_rows();
             if (nr_of_equ > 0) {
                 out << nr_of_equ <<" equations:" <<endl;
@@ -1160,7 +1187,9 @@ void Output<Integer>::write_files() const {
             }
             
             //lattice
-            const Matrix<Integer>& LatticeBasis = BasisChange.getEmbeddingMatrix();
+            const Matrix<Integer>& LB = BasisChange.getEmbeddingMatrix();
+            Matrix<Integer> LatticeBasis=LB;
+            LatticeBasis.row_echelon_reduce();
             size_t nr_of_latt = LatticeBasis.nr_of_rows();
             if (nr_of_latt < dim ||  BasisChange.getExternalIndex()!=1) {
                 out << nr_of_latt <<" basis elements of lattice:" <<endl;
